@@ -617,16 +617,15 @@ class TestResumableUpload(object):
         assert headers == expected_headers
 
     def test__prepare_request_success_with_headers(self):
-        headers = {"cannot": "touch this"}
+        headers = {"keep": "this"}
         new_headers = self._prepare_request_helper(headers)
         assert new_headers is not headers
         expected_headers = {
+            "keep": "this",
             "content-range": "bytes 0-32/33",
             "content-type": BASIC_CONTENT,
         }
         assert new_headers == expected_headers
-        # Make sure the ``_headers`` are not incorporated.
-        assert "cannot" not in new_headers
 
     @pytest.mark.parametrize("checksum", ["md5", "crc32c"])
     def test__prepare_request_with_checksum(self, checksum):
